@@ -22,14 +22,17 @@
  *
  */
 
-/*
- * @file hildon-color-selector.c
- *
- * This file contains the implementation of Hildon Color Selector
- * widget containing the base color and custom color palette selector 
- * and popup for selecting different colors based on RGB values.
+/**
+ * SECTION:hildon-color-selector
+ * @short_description: A widget for selecting a color
+ * @see_also: #HildonColorButton, #HildonColorPopup
+ * 
+ * #HildonColorSelector allows selection of a color from a standard
+ * 16-color palette or a palette of 8 user-customizable colors.
+ * The user-customizable colors can be modified through HildonColorPopup.
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -51,45 +54,59 @@
 /* Color amounts */
 #define HILDON_BASE_COLOR_NUM             16
 #define HILDON_CUSTOM_COLOR_NUM            8
-#define HILDON_TOTAL_COLOR_NUM \
-  (HILDON_BASE_COLOR_NUM + HILDON_CUSTOM_COLOR_NUM)
+#define HILDON_TOTAL_COLOR_NUM (HILDON_BASE_COLOR_NUM + HILDON_CUSTOM_COLOR_NUM)
 #define BLACKIND                           0
 #define GREYIND                            6
 #define WHITEIND                           9
 
+#define SELECTION_BORDER_COLOR_NAME        "ImageBorderColor"
+#define FOCUS_BORDER_COLOR                 "#8080FF"
+#define FOCUS_BORDER_WIDTH                 2
+
 /* Pixel sizes */
-#define HILDON_COLOR_SELECTOR_BOX_W       27
-#define HILDON_COLOR_SELECTOR_BOX_H       27
-#define HILDON_COLOR_SELECTOR_BOX_BORDER   2
-#define HILDON_COLOR_SELECTOR_COLS         8
-#define HILDON_COLOR_SELECTOR_ROWS         3
-#define HILDON_COLOR_PALETTE_SIZE        120
-#define HILDON_COLOR_CONTROLBAR_MAX       31
-#define HILDON_COLOR_CONTROLBAR_MIN        0 
-#define HILDON_COLOR_LABELS_LEFT_PAD      35
-#define HILDON_COLOR_PALETTE_POS_PAD      45
-#define HILDON_COLOR_BAR_WIDTH 449
+#define HILDON_COLOR_SELECTOR_BOX_W           26
+#define HILDON_COLOR_SELECTOR_BOX_H           26
+#define HILDON_COLOR_SELECTOR_BORDER_WIDTH     1
+#define HILDON_COLOR_SELECTOR_PADDING_WIDTH    1
+
+#define HILDON_COLOR_SELECTOR_BOX_FULL_WIDTH    \
+      ( HILDON_COLOR_SELECTOR_BOX_W             \
+      + HILDON_COLOR_SELECTOR_BORDER_WIDTH  * 2 \
+      + HILDON_COLOR_SELECTOR_PADDING_WIDTH * 2 )
+
+#define HILDON_COLOR_SELECTOR_BOX_FULL_HEIGHT   \
+      ( HILDON_COLOR_SELECTOR_BOX_H             \
+      + HILDON_COLOR_SELECTOR_BORDER_WIDTH  * 2 \
+      + HILDON_COLOR_SELECTOR_PADDING_WIDTH * 2 )
+
+#define HILDON_COLOR_SELECTOR_COLS             8
+#define HILDON_COLOR_SELECTOR_ROWS             3
 
 /* gconf definitions */
 #define HILDON_COLOR_GCONF_PATH  "/system/osso/af/color_selector"
 #define HILDON_COLOR_GCONF_KEYS  "/system/osso/af/color_selector/custom_colors"
-/*
- * Pointer parent class 
- */
+
+/* Pointer parent class */
 static GtkDialogClass *parent_class;
 
 struct _HildonColorSelectorPriv 
 {
     GConfClient *client;
-    GtkWidget *drawing_area;
-    GtkWidget *modify_button;
-    gint index;
-    guint notify_id;
-
+    GtkWidget   *drawing_area;
+    GtkWidget   *modify_button;
+    gint         selected_index;
+    gint         focused_index;
+    guint        notify_id;
     /* one extra place for the modified base color */
     GdkColor color[HILDON_TOTAL_COLOR_NUM + 1];
 };
 
+enum 
+{
+  PROP_NONE,
+  PROP_COLOR
+};
+
 /* 
  * Private function prototype definitions 
  */
@@ -100,36 +117,46 @@ static void
 hildon_color_selector_init (HildonColorSelector * selector);
 
 static gboolean
-hildon_color_selector_expose (GtkWidget * widget,
-                              GdkEventExpose * event, 
-                              gpointer data);
+hildon_color_selector_expose  (GtkWidget      * widget,
+                               GdkEventExpose * event, 
+                               gpointer         data);
 
-static gboolean 
-key_pressed (GtkWidget * widget, 
-             GdkEventKey * event);
+static gboolean key_pressed   (GtkWidget      * widget,
+                               GdkEventKey    * event);
 
-static gboolean 
-color_pressed (GtkWidget * widget, 
-               GdkEventButton * event, 
-               gpointer user_data);
+static gboolean color_pressed (GtkWidget      * widget,
+                               GdkEventButton * event,
+                               gpointer         user_data);
+
+static void     select_color_index (HildonColorSelector *selector,
+                                    gint                 idx,
+                                    gboolean             motion);
+
+static void     select_color  (HildonColorSelector * selector,
+                               int      event_x,
+                               int      event_y,
+                               gboolean motion);
+
+static gboolean color_moved   (GtkWidget      * widget,
+                               GdkEventMotion * event,
+                               gpointer         data);
 
 static void
-select_color (HildonColorSelector * selector, 
-              int event_x, 
-              int event_y,
-              gboolean motion);
+modify_button_clicked         (GtkWidget           * button,
+                               HildonColorSelector * selector);
 
-static gboolean
-color_moved (GtkWidget * widget, 
-             GdkEventMotion * event, 
-             gpointer data);
+static void modify_focused(HildonColorSelector * colselector);
 
 static void
-modify_button_clicked (GtkWidget * button,
-                       HildonColorSelector * selector);
-
+hildon_color_selector_set_property(GObject      *object,
+                                   guint         param_id,
+                                   const GValue *value,
+                                   GParamSpec   *pspec);
 static void
-modify_selected(HildonColorSelector * colselector);
+hildon_color_selector_get_property(GObject    *object,
+                                   guint       param_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec);
 
 GType
 hildon_color_selector_get_type(void)
@@ -141,15 +168,16 @@ hildon_color_selector_get_type(void)
         static const GTypeInfo selector_info = 
             {
                 sizeof(HildonColorSelectorClass),
-                NULL,       /* base_init */
-                NULL,       /* base_finalize */
+                NULL,       /* base_init      */
+                NULL,       /* base_finalize  */
                 (GClassInitFunc) hildon_color_selector_class_init,
                 NULL,       /* class_finalize */
-                NULL,       /* class_data */
+                NULL,       /* class_data     */
                 sizeof(HildonColorSelector),
-                0,  /* n_preallocs */
+                0,          /* n_preallocs    */
                 (GInstanceInitFunc) hildon_color_selector_init,
             };
+
         selector_type = g_type_register_static(GTK_TYPE_DIALOG,
                                                "HildonColorSelector",
                                                &selector_info, 0);
@@ -176,6 +204,7 @@ static void
 hildon_color_selector_class_init(HildonColorSelectorClass * selector_class)
 {
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(selector_class);
+    GObjectClass *gobject_class = G_OBJECT_CLASS (selector_class);
 
     parent_class = g_type_class_peek_parent(selector_class);
 
@@ -185,23 +214,39 @@ hildon_color_selector_class_init(HildonColorSelectorClass * selector_class)
                              sizeof(HildonColorSelectorPriv));
 
     GTK_OBJECT_CLASS(selector_class)->destroy = hildon_color_selector_destroy;
+  
+    gobject_class->get_property = hildon_color_selector_get_property;
+    gobject_class->set_property = hildon_color_selector_set_property;
+  
+    /**
+     * HildonColorSelector:color:
+     *
+     * The selected color.
+     */
+    g_object_class_install_property (gobject_class, PROP_COLOR,
+	    g_param_spec_boxed ("color",
+                "Current Color",
+                "The selected color",
+                GDK_TYPE_COLOR,
+                G_PARAM_READWRITE));
 }
 
 
 /**
  * hildon_color_selector_new:
- * @parent:  The parent window. The X window ID of the parent window
- *           has to be the same as the X window ID of the application.
- * @returns: new #HildonColorSelector.
+ * @parent:  the parent window. The X window ID of the parent window
+ *           has to be the same as the X window ID of the application
  *
  * Creates a new #HildonColorSelector dialog with 3x8 layout of 
- * Windows base colors and 'OK', 'More..' and 'Cancel' buttons.
+ * base colors and 'OK', 'More..' and 'Cancel' buttons.
+ *
+ * Returns: new #HildonColorSelector
  **/
 GtkWidget *hildon_color_selector_new(GtkWindow * parent)
 {
     GtkWidget *dialog = g_object_new(HILDON_TYPE_COLOR_SELECTOR, NULL);
 
-    g_assert(dialog);
+    g_return_val_if_fail(dialog, NULL);
 
     if (parent) 
     {
@@ -217,18 +262,20 @@ hildon_color_selector_set_custom_colors(
   GConfValue *value)
 {
   GSList *list;
-  int i;
+  gint i;
 
   g_assert(HILDON_IS_COLOR_SELECTOR(selector));
 
-  /* We have to be really carefull. At least gconftool's
+  /* We have to be really careful. At least gconftool's
      stress test may generate unexpected value setups */
-  if (value == NULL || value->type != GCONF_VALUE_LIST ||
-    gconf_value_get_list_type(value) != GCONF_VALUE_STRING)
+  if (value == NULL
+      || value->type != GCONF_VALUE_LIST
+      || gconf_value_get_list_type(value) != GCONF_VALUE_STRING)
     list = NULL;
   else
     list = gconf_value_get_list(value);
 
+  /* Use list to fill in the selector's color property */
   for ( i = 0; i < HILDON_CUSTOM_COLOR_NUM; ++i)
   {
     const gchar *color_string = NULL;
@@ -289,6 +336,7 @@ hildon_color_selector_init(HildonColorSelector * selector)
                               GCONF_CLIENT_PRELOAD_NONE,
                               NULL);
     
+    /* Use the value directed by GConfValue to set the color */
     value = gconf_client_get(selector->priv->client, 
       HILDON_COLOR_GCONF_KEYS, NULL);    
 
@@ -304,7 +352,8 @@ hildon_color_selector_init(HildonColorSelector * selector)
 
     /* ************************************  */
 
-    selector->priv->index = GREYIND;
+    selector->priv->selected_index = GREYIND;
+    selector->priv->focused_index  = GREYIND;
     
     /* init base colors for color boxes */
     for (i = 0; i < HILDON_BASE_COLOR_NUM; ++i) 
@@ -325,21 +374,19 @@ hildon_color_selector_init(HildonColorSelector * selector)
     gtk_widget_add_events (selector->priv->drawing_area,
                           GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK);
 
+    /* Arrange size of the drawing area. */
     gtk_widget_set_size_request (selector->priv->drawing_area,
-                                 (HILDON_COLOR_SELECTOR_BOX_W *
-                                  HILDON_COLOR_SELECTOR_COLS) +
-                                 (HILDON_COLOR_SELECTOR_BOX_BORDER *
-                                  2*HILDON_COLOR_SELECTOR_COLS),
-                                 (HILDON_COLOR_SELECTOR_BOX_H *
-                                  HILDON_COLOR_SELECTOR_ROWS) +
-                                 HILDON_COLOR_SELECTOR_BOX_BORDER *
-                                 2 * HILDON_COLOR_SELECTOR_ROWS);
+                                 (HILDON_COLOR_SELECTOR_COLS *
+                                  HILDON_COLOR_SELECTOR_BOX_FULL_WIDTH),
+                                 (HILDON_COLOR_SELECTOR_ROWS *
+                                  HILDON_COLOR_SELECTOR_BOX_FULL_HEIGHT));
 
     gtk_box_pack_start (GTK_BOX(GTK_DIALOG(selector)->vbox), 
                        hbox, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX(hbox), selector->priv->drawing_area,
                        FALSE, FALSE, 0);
 
+    /* Register callback functions for the drawing area */
     g_signal_connect (selector->priv->drawing_area, "expose_event",
                       G_CALLBACK(hildon_color_selector_expose), selector);
     g_signal_connect (selector->priv->drawing_area, "button_press_event", 
@@ -377,8 +424,8 @@ hildon_color_selector_expose(GtkWidget * widget,
 {
     HildonColorSelector *selector;
     GdkColor color;
-    GdkGC *gc;
-    gint x, y;
+    GdkGC *gc, *gc_focus;
+    gint x, y, idx;
 
     g_return_val_if_fail (GTK_IS_WIDGET(widget), FALSE);
     g_return_val_if_fail (event, FALSE);
@@ -390,57 +437,83 @@ hildon_color_selector_expose(GtkWidget * widget,
     selector = HILDON_COLOR_SELECTOR(data);
     gc = gdk_gc_new (widget->window); 
 
+    gc_focus = gdk_gc_new(widget->window);
+    gdk_gc_set_line_attributes(gc_focus, FOCUS_BORDER_WIDTH,
+                               GDK_LINE_SOLID, GDK_CAP_BUTT,
+                               GDK_JOIN_MITER);
+
     /* draw the color boxes and a focus for one of them */
-    for (x = 0; x < HILDON_COLOR_SELECTOR_COLS; ++x) 
+    for (y = 0, idx = 0; y < HILDON_COLOR_SELECTOR_ROWS; ++y)
     {
-        for (y = 0; y < HILDON_COLOR_SELECTOR_ROWS; ++y) 
+      for (x = 0; x < HILDON_COLOR_SELECTOR_COLS; ++x, ++idx)
         {
-            color.pixel = color.red = color.green = color.blue = 0;
-            gdk_gc_set_rgb_fg_color(gc, &color);
+            gint xpos = x * HILDON_COLOR_SELECTOR_BOX_FULL_WIDTH;
+            gint ypos = y * HILDON_COLOR_SELECTOR_BOX_FULL_HEIGHT;
 
-            /* focus around the selected color box */
-            if ( (y * HILDON_COLOR_SELECTOR_COLS + x) ==
-                 selector->priv->index) 
-            {
-                /* focus box */
-                gdk_draw_rectangle(widget->window, gc, TRUE,
-                                   (HILDON_COLOR_SELECTOR_BOX_BORDER * 2 +
-                                    HILDON_COLOR_SELECTOR_BOX_W) * x,
-                                   (HILDON_COLOR_SELECTOR_BOX_BORDER * 2 +
-                                    HILDON_COLOR_SELECTOR_BOX_H) * y,
-                                   HILDON_COLOR_SELECTOR_BOX_W +
-                                   HILDON_COLOR_SELECTOR_BOX_BORDER * 2,
-                                   HILDON_COLOR_SELECTOR_BOX_H +
-                                   HILDON_COLOR_SELECTOR_BOX_BORDER * 2); 
-            }
-            
             /* frames on color box */
-            gdk_draw_rectangle(widget->window, gc, FALSE,
-                               (HILDON_COLOR_SELECTOR_BOX_BORDER * 2 +
-                                HILDON_COLOR_SELECTOR_BOX_W) * x + 1,
-                               (HILDON_COLOR_SELECTOR_BOX_BORDER * 2 +
-                                HILDON_COLOR_SELECTOR_BOX_H) * y + 1,
-                               HILDON_COLOR_SELECTOR_BOX_W + 1,
-                               HILDON_COLOR_SELECTOR_BOX_H + 1);
+            gdk_draw_rectangle(widget->window, widget->style->black_gc, FALSE,
+                               xpos + HILDON_COLOR_SELECTOR_PADDING_WIDTH,
+                               ypos + HILDON_COLOR_SELECTOR_PADDING_WIDTH,
+                               HILDON_COLOR_SELECTOR_BOX_W +
+                               HILDON_COLOR_SELECTOR_PADDING_WIDTH,
+                               HILDON_COLOR_SELECTOR_BOX_H +
+                               HILDON_COLOR_SELECTOR_PADDING_WIDTH);
             
-            gdk_gc_set_rgb_fg_color(gc, 
-              &(selector->priv->color[y * HILDON_COLOR_SELECTOR_COLS + x]));
-
             /* color box */
+            gdk_gc_set_rgb_fg_color(gc, &(selector->priv->color[idx]));
             gdk_draw_rectangle(widget->window, gc,
                                TRUE,    /* filled */
-                               HILDON_COLOR_SELECTOR_BOX_BORDER +
-                               (HILDON_COLOR_SELECTOR_BOX_BORDER * 2 +
-                                HILDON_COLOR_SELECTOR_BOX_W) * x,
-                               HILDON_COLOR_SELECTOR_BOX_BORDER +
-                               (HILDON_COLOR_SELECTOR_BOX_BORDER * 2 +
-                                HILDON_COLOR_SELECTOR_BOX_H) * y,
+                               xpos + HILDON_COLOR_SELECTOR_PADDING_WIDTH +
+                               HILDON_COLOR_SELECTOR_BORDER_WIDTH,
+                               ypos + HILDON_COLOR_SELECTOR_PADDING_WIDTH +
+                               HILDON_COLOR_SELECTOR_BORDER_WIDTH,
                                HILDON_COLOR_SELECTOR_BOX_W,
                                HILDON_COLOR_SELECTOR_BOX_H);
+
+            /* focus around the selected and focused color box */
+            if (idx == selector->priv->selected_index)
+            {
+                /* selected color box */
+                if (!gtk_style_lookup_logical_color(widget->style,
+                                    SELECTION_BORDER_COLOR_NAME, &color))
+                {
+                    /* set default color if color looking up fails */
+                    color.red = color.green = color.blue = 0;
+                }
+
+                gdk_gc_set_rgb_fg_color(gc_focus, &color);
+                gdk_draw_rectangle(widget->window, gc_focus, FALSE,
+                                   xpos + 1-(FOCUS_BORDER_WIDTH % 2),
+                                   ypos + 1-(FOCUS_BORDER_WIDTH % 2),
+                                   HILDON_COLOR_SELECTOR_BOX_W +
+                                   HILDON_COLOR_SELECTOR_BORDER_WIDTH * 2 +
+                                   (FOCUS_BORDER_WIDTH % 2),
+                                   HILDON_COLOR_SELECTOR_BOX_H +
+                                   HILDON_COLOR_SELECTOR_BORDER_WIDTH * 2 +
+                                   (FOCUS_BORDER_WIDTH % 2));
+            }
+            else if (idx == selector->priv->focused_index)
+            {
+                /* focused color box.
+                   There's only zero or one so allocate GC in here. */
+                gdk_color_parse(FOCUS_BORDER_COLOR, &color);
+
+                gdk_gc_set_rgb_fg_color(gc_focus, &color);
+                gdk_draw_rectangle(widget->window, gc_focus, FALSE,
+                                   xpos + 1-(FOCUS_BORDER_WIDTH % 2),
+                                   ypos + 1-(FOCUS_BORDER_WIDTH % 2),
+                                   HILDON_COLOR_SELECTOR_BOX_W +
+                                   HILDON_COLOR_SELECTOR_BORDER_WIDTH * 2 +
+                                   (FOCUS_BORDER_WIDTH % 2),
+                                   HILDON_COLOR_SELECTOR_BOX_H +
+                                   HILDON_COLOR_SELECTOR_BORDER_WIDTH * 2 +
+                                   (FOCUS_BORDER_WIDTH % 2));
+            }
         }
     }
 
     g_object_unref(gc);
+    g_object_unref(gc_focus);
     return TRUE;
 }
 
@@ -448,23 +521,22 @@ hildon_color_selector_expose(GtkWidget * widget,
  * hildon_color_selector_get_color:
  * @selector: a #HildonColorSelector
  * 
- * Gets the currently selected base color as #GdkColor.
- * 
- * Returns: A copy of the currently selected #GdkColor.
+ * Returns: the currently selected #GdkColor. The returned pointer must 
+ * not be freed.
  */
-GdkColor *hildon_color_selector_get_color(HildonColorSelector * selector)
+G_CONST_RETURN GdkColor *hildon_color_selector_get_color(HildonColorSelector * selector)
 {
     g_return_val_if_fail(HILDON_IS_COLOR_SELECTOR(selector), NULL);    
-    return &(selector->priv->color[selector->priv->index]);
+    return &(selector->priv->color[selector->priv->selected_index]);
 }
 
 /**
  * hildon_color_selector_set_color:
  * @selector: #HildonColorSelector
- * @color: #Gdkcolor to set.
+ * @color: #Gdkcolor to set
  * 
- * Select the color specified. Does nothing if the color does not 
- * exists among the standard colors.
+ * Selects the color specified. Does nothing if the color does not 
+ * exist among the standard colors.
  */
 void hildon_color_selector_set_color(HildonColorSelector * selector,
                                      GdkColor * color)
@@ -474,6 +546,7 @@ void hildon_color_selector_set_color(HildonColorSelector * selector,
     g_return_if_fail(HILDON_IS_COLOR_SELECTOR(selector));
     g_return_if_fail(color);
 
+    /* Select the specified color */
     for (i = 0; 
          i < (HILDON_BASE_COLOR_NUM + HILDON_CUSTOM_COLOR_NUM);
          ++i) 
@@ -482,13 +555,21 @@ void hildon_color_selector_set_color(HildonColorSelector * selector,
             selector->priv->color[i].green == color->green &&
             selector->priv->color[i].blue  == color->blue) 
         {
-            selector->priv->index = i;
+            selector->priv->selected_index = i;
+            selector->priv->focused_index  = i;
+
+            /* The modify button is active if the color index is bigger than
+             * the number of base colours.
+             */
             gtk_widget_set_sensitive(selector->priv->modify_button,
-                                     selector->priv->index >= HILDON_BASE_COLOR_NUM);
+                                     selector->priv->focused_index >= HILDON_BASE_COLOR_NUM);
             gtk_widget_queue_draw(selector->priv->drawing_area);
             break;
         }
     }
+
+    /* Notify the color selector that the color has changed */  
+    g_object_notify (G_OBJECT (selector), "color");
 }
 
 static gboolean
@@ -499,23 +580,25 @@ color_pressed(GtkWidget * widget, GdkEventButton * event,
     return TRUE;
 }
 
+/* Handle key press of right, left, up, down and return */
 static gboolean key_pressed(GtkWidget * widget,
                             GdkEventKey * event)
 {
     HildonColorSelector *selector;
     gint index;
 
-    g_return_val_if_fail(widget, FALSE);
+    g_assert(widget);
 
     selector = HILDON_COLOR_SELECTOR(widget);
-    index = selector->priv->index;
+    index = selector->priv->focused_index;
 
     /* if dialog buttons has the focus */
     if (GTK_WIDGET_HAS_FOCUS(selector->priv->drawing_area) == FALSE)
         return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget, event);
 
-    /* go for if available index otherwise stop keypress handler because
-       wrapping around is not allowed. */
+    /* Since wrapping is not allowed,
+     * move only if the next index is a valid one.
+     */
     switch (event->keyval) {
     case GDK_KP_Right:
     case GDK_Right:
@@ -562,24 +645,27 @@ static gboolean key_pressed(GtkWidget * widget,
     case GDK_KP_Enter:
     case GDK_Return:
         if (index < HILDON_BASE_COLOR_NUM)
-            return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget, event);
-
-        modify_selected(selector);
+            select_color_index(selector, selector->priv->focused_index, FALSE);
+        else
+            modify_focused(selector);
     default:
         return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget, event);
     }
 
     if (index < (HILDON_BASE_COLOR_NUM + HILDON_CUSTOM_COLOR_NUM)) 
     {
-        selector->priv->index = index;
+        selector->priv->focused_index = index;
     } 
     else 
     {
-        selector->priv->index = 
+        selector->priv->focused_index =
             HILDON_BASE_COLOR_NUM + HILDON_CUSTOM_COLOR_NUM - 1;
     }
+    /* The modify button is active if the color index is bigger than
+     * the number of base colours.
+     */
     gtk_widget_set_sensitive(selector->priv->modify_button,
-                             selector->priv->index >= HILDON_BASE_COLOR_NUM);
+                             selector->priv->focused_index >= HILDON_BASE_COLOR_NUM);
 
     gtk_widget_queue_draw(selector->priv->drawing_area);
     
@@ -587,23 +673,34 @@ static gboolean key_pressed(GtkWidget * widget,
 }
 
 static void
-select_color(HildonColorSelector * selector, int event_x, int event_y,
+select_color_index(HildonColorSelector * selector, gint idx, gboolean motion)
+{
+    /* If a custom colour is selected, open a popup to modify the colour */ 
+    if (!motion &&
+        selector->priv->focused_index >= HILDON_BASE_COLOR_NUM &&
+        selector->priv->focused_index == idx)
+        modify_focused(selector);
+
+    selector->priv->focused_index = selector->priv->selected_index = idx;
+    gtk_widget_set_sensitive(selector->priv->modify_button,
+                             selector->priv->focused_index >= HILDON_BASE_COLOR_NUM);
+    
+    gtk_widget_queue_draw(selector->priv->drawing_area);
+}
+
+static void
+select_color(HildonColorSelector * selector, gint event_x, gint event_y,
              gboolean motion)
 {
     gint x, y;
 
-    g_return_if_fail(HILDON_IS_COLOR_SELECTOR(selector));
+    g_assert(HILDON_IS_COLOR_SELECTOR(selector));
 
-    x = ( (event_x - HILDON_COLOR_SELECTOR_BOX_BORDER) /
-          (HILDON_COLOR_SELECTOR_BOX_BORDER * 2 +
-           HILDON_COLOR_SELECTOR_BOX_W)
-        );
-    y = ( (event_y -
-           HILDON_COLOR_SELECTOR_BOX_BORDER) /
-          (HILDON_COLOR_SELECTOR_BOX_BORDER * 2 +
-           HILDON_COLOR_SELECTOR_BOX_H)
-        );
+    /* Get the selection coordinates */ 
+    x = event_x / HILDON_COLOR_SELECTOR_BOX_FULL_WIDTH;
+    y = event_y / HILDON_COLOR_SELECTOR_BOX_FULL_HEIGHT;
 
+    /* Get the row and column numbers for the selected color */ 
     if (x > (HILDON_COLOR_SELECTOR_COLS + HILDON_CUSTOM_COLOR_NUM - 1)) 
     {
         x = HILDON_COLOR_SELECTOR_COLS + HILDON_CUSTOM_COLOR_NUM - 1;
@@ -621,16 +718,7 @@ select_color(HildonColorSelector * selector, int event_x, int event_y,
         y = 0;
     }
 
-    if (!motion &&
-        selector->priv->index >= HILDON_BASE_COLOR_NUM &&
-        selector->priv->index == (x + y * HILDON_COLOR_SELECTOR_COLS))
-        modify_selected(selector);
-
-    selector->priv->index = (x + y * HILDON_COLOR_SELECTOR_COLS);
-    gtk_widget_set_sensitive(selector->priv->modify_button,
-                             selector->priv->index >= HILDON_BASE_COLOR_NUM);
-    
-    gtk_widget_queue_draw(selector->priv->drawing_area);
+    select_color_index(selector, (x + y * HILDON_COLOR_SELECTOR_COLS), motion);
 }
 
 static gboolean
@@ -648,15 +736,16 @@ color_moved(GtkWidget * widget, GdkEventMotion * event, gpointer data)
 static void
 modify_button_clicked(GtkWidget * button, HildonColorSelector * selector)
 {
-    modify_selected (selector);
+    modify_focused (selector);
 }
 
 static void
-modify_selected(HildonColorSelector * colselector)
+modify_focused(HildonColorSelector * colselector)
 {
     HildonColorPopup popupdata;
     GtkWidget *popup;
     
+    /* Create a HildonColorPopup dialog */     
     popup = hildon_color_popup_new(GTK_WINDOW(colselector), 
 		    hildon_color_selector_get_color(colselector), &popupdata);
     
@@ -665,18 +754,19 @@ modify_selected(HildonColorSelector * colselector)
       GdkColor *color;
 
       /* We cannot modify a base color */
-      if (colselector->priv->index < HILDON_BASE_COLOR_NUM)
+      if (colselector->priv->focused_index < HILDON_BASE_COLOR_NUM)
       {
         colselector->priv->color[HILDON_TOTAL_COLOR_NUM] = 
-          colselector->priv->color[colselector->priv->index];
-        colselector->priv->index = HILDON_TOTAL_COLOR_NUM;
+          colselector->priv->color[colselector->priv->focused_index];
+        colselector->priv->focused_index = HILDON_TOTAL_COLOR_NUM;
       }
 
-      color = hildon_color_selector_get_color(colselector);
+      /* Update the focused color with the color selected in color popup */
+      color = &(colselector->priv->color[colselector->priv->focused_index]);
       hildon_color_popup_set_color_from_sliders(color, &popupdata);
 
       /* If we modified a base color we just accept the dialog */      
-      if( colselector->priv->index >=  HILDON_TOTAL_COLOR_NUM)
+      if( colselector->priv->focused_index >=  HILDON_TOTAL_COLOR_NUM)
       {          
         gtk_dialog_response(GTK_DIALOG(colselector), GTK_RESPONSE_OK); 
       }
@@ -684,7 +774,7 @@ modify_selected(HildonColorSelector * colselector)
       {
         GConfValue *value;
         GSList * list;
-        int i;  
+        gint i;  
 
         value = gconf_value_new(GCONF_VALUE_LIST);
         gconf_value_set_list_type(value, GCONF_VALUE_STRING);
@@ -718,3 +808,38 @@ modify_selected(HildonColorSelector * colselector)
     gtk_widget_destroy (popup); 
     gtk_window_present (GTK_WINDOW(colselector));
 }
+
+static void
+hildon_color_selector_set_property(GObject *object, guint param_id,
+			                           const GValue *value, GParamSpec *pspec)
+{
+  HildonColorSelector *selector = HILDON_COLOR_SELECTOR(object);
+
+  switch (param_id) 
+    {
+    case PROP_COLOR:
+      hildon_color_selector_set_color(selector, (GdkColor*)g_value_get_boxed(value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
+      break;
+    }
+}
+
+static void
+hildon_color_selector_get_property(GObject *object, guint param_id,
+			                           GValue *value, GParamSpec *pspec)
+{
+  HildonColorSelector *selector = HILDON_COLOR_SELECTOR(object);
+
+  switch (param_id) 
+    {
+    case PROP_COLOR:
+      g_value_set_boxed(value, (gconstpointer)hildon_color_selector_get_color(selector));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
+      break;
+    }
+}
+
