@@ -22,13 +22,17 @@
  *
  */
 
-/*
- * @file hildon-caption-control.c
- *
- * This file contains the implementation of Hildon Caption Control
+/**
+ * SECTION:hildon-controlbar
+ * @short_description: A widget that allows increasing or decreasing
+ * a value within a pre-defined range
  *
+ * #HildonControlbar is a horizontally positioned range widget that is
+ * visually divided into blocks and supports setting a minimum and
+ * maximum value for the range.
  */
 
+
 #include <math.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
@@ -100,6 +104,11 @@ static void hildon_controlbar_get_property( GObject *object, guint param_id,
 static void
 hildon_controlbar_value_changed( GtkAdjustment *adj, GtkRange *range );
 
+/*
+ * Purpose of this function is to prevent Up and Down keys from 
+ * changing the widget's value (like Left and Right). Instead they 
+ * are used for changing focus to other widgtes.
+ */
 static gboolean
 hildon_controlbar_change_value( GtkRange *range, GtkScrollType scroll,
                                 gdouble new_value, gpointer data );
@@ -216,6 +225,7 @@ static void hildon_controlbar_init(HildonControlbar * controlbar)
     GtkRange *range;
     HildonControlbarPrivate *priv;
 
+    /* Initialize the private property */
     priv = HILDON_CONTROLBAR_GET_PRIVATE(controlbar);
     priv->button_press = FALSE;
     priv->old_value = 0;
@@ -242,6 +252,7 @@ static GObject *hildon_controlbar_constructor(GType type,
    
   gtk_scale_set_draw_value (GTK_SCALE (obj), FALSE);
 
+  /* Initialize the GtkAdjustment of the controlbar*/
   adj = GTK_RANGE(obj)->adjustment;
   adj->step_increment = HILDON_CONTROLBAR_STEP_INCREMENT;
   adj->page_increment = HILDON_CONTROLBAR_PAGE_INCREMENT;
@@ -306,6 +317,9 @@ hildon_controlbar_value_changed( GtkAdjustment *adj, GtkRange *range )
 {
   HildonControlbarPrivate *priv = HILDON_CONTROLBAR_GET_PRIVATE(range);
 
+  /* Change the controlbar value if the adjusted value is large enough 
+   * otherwise, keep the old value
+   */
   if( ABS(ceil(adj->value) - priv->old_value) >= 1 )
   {
     priv->old_value = ceil(adj->value);
@@ -321,14 +335,18 @@ hildon_controlbar_value_changed( GtkAdjustment *adj, GtkRange *range )
  * 
  * Creates a new #HildonControlbar widget.
  *
- * Return value: A #GtkWidget pointer of newly created #HildonControlbar
- * widget.
+ * Returns: a #GtkWidget pointer of newly created control bar
+ * widget
  */
 GtkWidget *hildon_controlbar_new(void)
 {
     return GTK_WIDGET(g_object_new(HILDON_TYPE_CONTROLBAR, NULL));
 }
 
+/* This function prevents Up and Down keys from changing the
+ * widget's value (like Left and Right).
+ * Instead they are used for changing focus to other widgtes.
+ */
 static gboolean
 hildon_controlbar_keypress(GtkWidget * widget, GdkEventKey * event)
 {
@@ -349,11 +367,10 @@ hildon_controlbar_size_request(GtkWidget * self, GtkRequisition * req)
 
 /**
  * hildon_controlbar_set_value:
- * @self: Pointer to #HildonControlbar
- * @value: Value in range of >= 0 && < G_MAX_INT
+ * @self: pointer to #HildonControlbar
+ * @value: value in range of >= 0 && < G_MAX_INT
  *
- * This method will change #HildonControlbar current value
- * to the given value.
+ * Change the current value of the control bar to the specified value.
  */
 void hildon_controlbar_set_value(HildonControlbar * self, gint value)
 {
@@ -376,11 +393,9 @@ void hildon_controlbar_set_value(HildonControlbar * self, gint value)
 
 /**
  * hildon_controlbar_get_value:
- * @self: Pointer to #HildonControlbar
+ * @self: pointer to #HildonControlbar
  *
- * Accessor method for getting controlbars current value
- *
- * Return value: Current value as gint.
+ * Returns: current value as gint
  */
 gint hildon_controlbar_get_value(HildonControlbar * self)
 {
@@ -393,16 +408,14 @@ gint hildon_controlbar_get_value(HildonControlbar * self)
 
 /**
  * hildon_controlbar_set_max:
- * @self: Pointer to #HildonControlbar
- * @max: Maximum value to set. The value needs to be
- * in range of > 0 && <= G_MAX_INT.
- * If the new maximum is smaller than current value,
- * the value will be adjusted so that
- * it equals the new maximum.
+ * @self: pointer to #HildonControlbar
+ * @max: maximum value to set. The value needs to be greater than 0.
  *
- * Accessor method for setting #HildonControlbar maximum to the
- * given value.
- **/
+ * Set the control bar's maximum to the given value.
+ * 
+ * If the new maximum is smaller than current value, the value will be
+ * adjusted so that it equals the new maximum.
+ */
 void hildon_controlbar_set_max(HildonControlbar * self, gint max)
 {
     GtkAdjustment *adj;
@@ -423,16 +436,15 @@ void hildon_controlbar_set_max(HildonControlbar * self, gint max)
 
 /**
  * hildon_controlbar_set_min:
- * @self: Pointer to #HildonControlbar
- * @min: Minimum value to set. The value needs to be
- * in range of => 0 && <= G_MAX_INT.
- * If the new minimum is smaller than current value,
- * the value will be adjusted so that
- * it equals the new minimum.
+ * @self: pointer to #HildonControlbar
+ * @min: minimum value to set. The value needs to be greater than or
+ * equal to 0.
  *
- * Accessor method for setting #HildonControlbar minimum to the
- * given value.
- **/
+ * Set the control bar's minimum to the given value.
+ *
+ * If the new minimum is smaller than current value, the value will be
+ * adjusted so that it equals the new minimum.
+ */
 void hildon_controlbar_set_min(HildonControlbar * self, gint min)
 {
     GtkAdjustment *adj;
@@ -452,21 +464,19 @@ void hildon_controlbar_set_min(HildonControlbar * self, gint min)
 
 /**
  * hildon_controlbar_set_range:
- * @self: Pointer to #HildonControlbar
- * @max: Maximum value to set. The value needs to be
- * in range of > 0 && <= G_MAX_INT.
- * @min: Minimum value to set. The value needs to be
- * in range of => 0 && <= G_MAX_INT.
- * If the new maximum is smaller than current value,
- * the value will be adjusted so that
- * it equals the new maximum.
- * If the new minimum is smaller than current value,
- * the value will be adjusted so that
- * it equals the new minimum.
+ * @self: pointer to #HildonControlbar
+ * @max: maximum value to set. The value needs to be greater than 0.
+ * @min: Minimum value to set. The value needs to be greater than or
+ * equal to 0.
  *
- * Accessor method for setting #HildonControlbar maximum to the
- * given value.
- **/
+ * Set the controlbars range to the given value
+ * 
+ * If the new maximum is smaller than current value, the value will be
+ * adjusted so that it equals the new maximum.
+ *
+ * If the new minimum is smaller than current value, the value will be
+ * adjusted so that it equals the new minimum.
+ */
 void hildon_controlbar_set_range(HildonControlbar * self, gint min,
                                  gint max)
 {
@@ -483,11 +493,9 @@ void hildon_controlbar_set_range(HildonControlbar * self, gint min,
 
 /**
  * hildon_controlbar_get_max:
- * @self: A pointer to #HildonControlbar
+ * @self: a pointer to #HildonControlbar
  *
- * Accessor method for getting controlbars current maximum value
- *
- * Return value: Maximum value as gint
+ * Returns: maximum value of control bar
  */
 gint hildon_controlbar_get_max(HildonControlbar * self)
 {
@@ -500,11 +508,9 @@ gint hildon_controlbar_get_max(HildonControlbar * self)
 
 /**
  * hildon_controlbar_get_min:
- * @self: A pointer to #HildonControlbar
+ * @self: a pointer to #HildonControlbar
  *
- * Accessor method for getting controlbars current minimum value
- *
- * Return value: Minimum value as gint
+ * Returns: minimum value of controlbar
  */
 gint hildon_controlbar_get_min(HildonControlbar * self)
 {
@@ -557,15 +563,23 @@ static gint hildon_controlbar_button_press_event(GtkWidget * widget,
     return result;
 }
 
+/*
+ * Purpose of this function is to prevent Up and Down keys from 
+ * changing the widget's value (like Left and Right). Instead they 
+ * are used for changing focus to other widgtes.
+ */
 static gboolean
-hildon_controlbar_change_value( GtkRange *range, GtkScrollType scroll,
-                                gdouble new_value, gpointer data )
+hildon_controlbar_change_value( GtkRange      *range,
+				GtkScrollType  scroll,
+                                gdouble        new_value,
+                                gpointer       data )
 {
   HildonControlbarPrivate *priv;
   GtkAdjustment *adj = range->adjustment;
 
   priv = HILDON_CONTROLBAR_GET_PRIVATE(range);
 
+  /* Emit a signal when upper or lower limit is reached */
   switch (scroll)
   {
     case GTK_SCROLL_STEP_FORWARD :
