@@ -29,19 +29,29 @@
 
 G_BEGIN_DECLS
 /**
- * HILDON_DATE_EDITOR_TYPE:
+ * HILDON_TYPE_DATE_EDITOR:
  *
  * Macro for getting type of date editor.
+ * Since: 0.12.10
  */
-#define HILDON_DATE_EDITOR_TYPE ( hildon_date_editor_get_type() )
+#define HILDON_TYPE_DATE_EDITOR ( hildon_date_editor_get_type() )
+
+
+/**
+ * HILDON_DATE_EDITOR_TYPE:
+ *
+ * Deprecated: use #HILDON_TYPE_DATE_EDITOR instead
+ */
+#define HILDON_DATE_EDITOR_TYPE HILDON_TYPE_DATE_EDITOR
+
 #define HILDON_DATE_EDITOR(obj) (GTK_CHECK_CAST (obj,\
-  HILDON_DATE_EDITOR_TYPE, HildonDateEditor))
+  HILDON_TYPE_DATE_EDITOR, HildonDateEditor))
 #define HILDON_DATE_EDITOR_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass),\
-  HILDON_DATE_EDITOR_TYPE, HildonDateEditorClass))
+  HILDON_TYPE_DATE_EDITOR, HildonDateEditorClass))
 #define HILDON_IS_DATE_EDITOR(obj) (GTK_CHECK_TYPE (obj,\
-  HILDON_DATE_EDITOR_TYPE))
+  HILDON_TYPE_DATE_EDITOR))
 #define HILDON_IS_DATE_EDITOR_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass),\
-  HILDON_DATE_EDITOR_TYPE))
+  HILDON_TYPE_DATE_EDITOR))
 
 /**
  * HildonDateEditor:
@@ -53,6 +63,7 @@ typedef struct _HildonDateEditorClass HildonDateEditorClass;
 
 typedef enum
 {
+  NO_ERROR = -1,
   MAX_DAY,
   MAX_MONTH,
   MAX_YEAR,
@@ -77,7 +88,7 @@ struct _HildonDateEditorClass {
 			    HildonDateEditorErrorType type);
 };
 
-GType hildon_date_editor_get_type(void);
+GType hildon_date_editor_get_type(void) G_GNUC_CONST;
 
 GtkWidget *hildon_date_editor_new(void);
 
