@@ -22,12 +22,25 @@
  *
  */
 
-/*
- * @file hildon-grid.c
+/**
+ * SECTION:hildon-grid
+ * @short_description: Being used where ever a number of single tap
+ * activatable items need to be presented (e.g. Control Panel applets)
+ * @see_also: #HildonGridItem
  *
- * This file contains the implementation of HildonGrid. HildonGrid is used
- * in views like Home and Control Panel which have single-tap activated
- * items.
+ * HildonGrid is a set of application-defineable items that are presented in a
+ * table. There are two modes for the form of the table; large icon mode
+ * and small icon mode.
+ *
+ * In large icon mode, the Grid View items are presented with a large
+ * icon and a label under it. In small icon mode, the items are
+ * presented with a small icon and a label on the right side of the
+ * icon.
+ *
+ * The label has a solid background as wide as the maximum text width.
+ * This allows the text to have focus as well as be legible when
+ * displayed upon a black or dark background image. Long names are
+ * truncated with an ellipsis ("...") appended.
  */
 
 /*
@@ -56,6 +69,7 @@
 
 #include "hildon-grid-item-private.h"
 #include "hildon-marshalers.h"
+#include "hildon-app.h"
 #include <hildon-widgets/hildon-grid.h>
 #include <hildon-widgets/hildon-grid-item.h>
 
@@ -83,7 +97,9 @@ enum {
 
 enum {
     PROP_0,
-    PROP_EMPTY_LABEL
+    PROP_EMPTY_LABEL,
+    PROP_STYLE,
+    PROP_SCROLLBAR_POS
 };
 
 
@@ -280,11 +296,26 @@ static void hildon_grid_class_init(HildonGridClass * klass)
     container_class->child_type = hildon_grid_child_type;
     container_class->set_focus_child = hildon_grid_set_focus_child;
 
+    /* Install properties to the class */
     g_object_class_install_property(gobject_class, PROP_EMPTY_LABEL,
         g_param_spec_string("empty_label",
                             "Empty label",
                             "Label to show when grid has no items",
-                            _("Ckct_wi_grid_no_items"), G_PARAM_READWRITE));
+                            _("ckct_wi_grid_no_items"), G_PARAM_READWRITE));
+
+    g_object_class_install_property(gobject_class, PROP_STYLE,
+        g_param_spec_string("style",
+                            "Style",
+                            "Widget's Style. Setting style sets widget size, "
+			    "spacing, label position, number of columns, "
+			    "and icon sizeLabel to show when grid has no items",
+                            DEFAULT_STYLE, G_PARAM_READWRITE));
+
+    g_object_class_install_property(gobject_class, PROP_SCROLLBAR_POS,
+        g_param_spec_int("scrollbar-position",
+                         "Scrollbar Position",
+                         "View (scrollbar) position.",
+                         G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
 
     gtk_widget_class_install_style_property(widget_class,
         g_param_spec_uint("item_width",
@@ -377,10 +408,9 @@ static void hildon_grid_class_init(HildonGridClass * klass)
                      G_OBJECT_CLASS_TYPE(gobject_class),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET(HildonGridClass, popup_context_menu),
-                     g_signal_accumulator_true_handled, NULL,
-                     _hildon_marshal_BOOLEAN__INT_INT_INT,
-                     G_TYPE_BOOLEAN, 3,
-                     G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);
+                     NULL, NULL,
+                     _hildon_marshal_VOID__OBJECT,
+                     G_TYPE_NONE, 1, HILDON_TYPE_GRID_ITEM);
 }
 
 
@@ -405,9 +435,7 @@ hildon_grid_set_empty_label(HildonGrid * grid, const gchar * empty_label)
  * _hildon_grid_get_empty_label:
  * @grid:   #HildonGrid
  *
- * Returns empty label. Label must not be modified nor freed.
- *
- * Return value: Label
+ * Returns: empty label. Label must not be modified nor freed.
  */
 static const gchar *
 hildon_grid_get_empty_label(HildonGrid * grid)
@@ -419,7 +447,7 @@ hildon_grid_get_empty_label(HildonGrid * grid)
 /*
  * hildon_grid_set_num_columns:
  * @grid:       #HildonGrid
- * @columsn:    Number of columns
+ * @columns:    Number of columns
  *
  * Sets number of columns.
  */
@@ -474,6 +502,7 @@ hildon_grid_set_label_pos(HildonGrid * grid,
 
     priv->label_pos = label_pos;
 
+    /* Set label position to each HildonGridItem */
     for (list = priv->children; list != NULL; list = list->next) {
         child = ((HildonGridChild *) list->data)->widget;
 
@@ -632,7 +661,7 @@ static void hildon_grid_init(HildonGrid * grid)
     priv->focus_index = -1;
 
     priv->scrollbar = gtk_vscrollbar_new(NULL);
-    priv->empty_label = gtk_label_new(_("Ckct_wi_grid_no_items"));
+    priv->empty_label = gtk_label_new(_("ckct_wi_grid_no_items"));
     priv->style = NULL;
 
     priv->area_height = 1;
@@ -678,7 +707,7 @@ static void hildon_grid_init(HildonGrid * grid)
  *
  * Creates a new #HildonGrid.
  *
- * Return value: A new #HildonGrid
+ * Returns: a new #HildonGrid
  */
 GtkWidget *hildon_grid_new(void)
 {
@@ -828,6 +857,8 @@ hildon_grid_expose(GtkWidget * widget, GdkEventExpose * event)
     priv = HILDON_GRID_GET_PRIVATE(grid);
     container = GTK_CONTAINER(grid);
 
+    /* If grid has no children,
+     * propagate the expose event to the label is one exists */ 
     if (priv->children == NULL || g_list_length(priv->children) == 0) {
         if (priv->empty_label != NULL) {
             gtk_container_propagate_expose(container,
@@ -1071,8 +1102,8 @@ hildon_grid_size_allocate(GtkWidget * widget, GtkAllocation * allocation)
 
 /**
  * hildon_grid_add:
- * @container:  Container (#HildonGrid) to add HildonGridItem into.
- * @widget:     #GtkWidget (#HildonGridItem) to add.
+ * @container:  container (#HildonGrid) to add HildonGridItem into
+ * @widget:     #GtkWidget (#HildonGridItem) to add
  *
  * Adds a new HildonGridItem into HildonGrid.
  */
@@ -1097,14 +1128,12 @@ static void hildon_grid_add(GtkContainer * container, GtkWidget * widget)
     }
     child->widget = widget;
 
-    _hildon_grid_item_set_label_pos(HILDON_GRID_ITEM(widget),
-                                    priv->label_pos);
-    _hildon_grid_item_set_focus_margin(HILDON_GRID_ITEM(widget),
-                                       priv->focus_margin);
-    _hildon_grid_item_set_icon_width(HILDON_GRID_ITEM(widget),
-                                     priv->icon_width);
-    _hildon_grid_item_set_emblem_size(HILDON_GRID_ITEM(widget),
-                                      priv->emblem_size);
+    _hildon_grid_item_set_label_pos   (HILDON_GRID_ITEM(widget), priv->label_pos);
+    _hildon_grid_item_set_focus_margin(HILDON_GRID_ITEM(widget), priv->focus_margin);
+    _hildon_grid_item_set_icon_width  (HILDON_GRID_ITEM(widget), priv->icon_width);
+    _hildon_grid_item_set_emblem_size (HILDON_GRID_ITEM(widget), priv->emblem_size);
+
+    /* Add the new item to the grid */
     priv->children = g_list_append(priv->children, child);
     gtk_widget_set_parent(widget, GTK_WIDGET(grid));
 
@@ -1139,8 +1168,8 @@ static void hildon_grid_add(GtkContainer * container, GtkWidget * widget)
 
 /**
  * hildon_grid_remove:
- * @container:  Container (#HildonGrid) to remove #HildonGridItem from.
- * @widget:     Widget (#HildonGridItem) to be removed.
+ * @container:  container (#HildonGrid) to remove #HildonGridItem from
+ * @widget:     widget (#HildonGridItem) to be removed
  *
  * Removes HildonGridItem from HildonGrid.
  */
@@ -1169,6 +1198,8 @@ hildon_grid_remove(GtkContainer * container, GtkWidget * widget)
          list != NULL; list = list->next, index++) {
         child = (HildonGridChild *) list->data;
         child_widget = child->widget;
+
+	/* Remove the Item if it is found in the grid */
         if (child_widget == widget) {
             gtk_widget_unparent(child_widget);
             priv->children = g_list_remove_link(priv->children, list);
@@ -1181,6 +1212,7 @@ hildon_grid_remove(GtkContainer * container, GtkWidget * widget)
         }
     }
 
+    /* Emit warning if the item is not found */
     if (!deleted) {
         g_warning("tried to remove unexisting item");
         return;
@@ -1248,6 +1280,7 @@ set_focus(HildonGrid * grid, GtkWidget * widget, gboolean refresh_view)
     priv = HILDON_GRID_GET_PRIVATE(grid);
     container = GTK_CONTAINER(grid);
 
+    /* If widget is NULL -> unfocus */ 
     if (widget == NULL && container->focus_child != NULL)
         GTK_WIDGET_UNSET_FLAGS(container->focus_child, GTK_HAS_FOCUS);
 
@@ -1257,6 +1290,7 @@ set_focus(HildonGrid * grid, GtkWidget * widget, gboolean refresh_view)
         return;
     }
 
+    /* Get the child index which the user wanted to focus */
     priv->focus_index = get_child_index(priv, widget);
 
     gtk_widget_grab_focus(widget);
@@ -1288,6 +1322,7 @@ hildon_grid_forall(GtkContainer * container,
     grid = HILDON_GRID(container);
     priv = HILDON_GRID_GET_PRIVATE(grid);
 
+    /* Connect callback functions */
     if (include_internals) {
         if (priv->scrollbar != NULL) {
             (*callback) (priv->scrollbar, callback_data);
@@ -1356,8 +1391,10 @@ static void hildon_grid_finalize(GObject * object)
  * . 1 # 2      . 1 # #     1 # # #     1 # # #
  * . . .        . . 2       . . .       . 2 .
  *
- * '.' = item, '#' = dimmed item, 
- * '1' = starting position, '2' = final position
+ * '.' = item,
+ * '#' = dimmed item, 
+ * '1' = starting position,
+ * '2' = final position
  *
  * ...although only the first example is implemented right now.
  *
@@ -1623,8 +1660,6 @@ hildon_grid_button_pressed(GtkWidget * widget,
     if (!GTK_WIDGET_IS_SENSITIVE(child))
         return FALSE;
 
-    set_focus(grid, child, TRUE);
-
     priv->click_x = event->x;
     priv->click_y = event->y;
 
@@ -1649,6 +1684,7 @@ hildon_grid_button_released(GtkWidget * widget,
     HildonGridPrivate *priv;
     GtkWidget *child;
     int child_no;
+    gboolean already_selected;
 
     grid = HILDON_GRID(widget);
     priv = HILDON_GRID_GET_PRIVATE(grid);
@@ -1674,9 +1710,16 @@ hildon_grid_button_released(GtkWidget * widget,
         && abs(priv->click_y - event->y) >= DRAG_SENSITIVITY) {
         return FALSE;
     }
+
+    /* Check if this element was already selected */
+    already_selected = (priv->focus_index == child_no);
+
     set_focus(grid, child, TRUE);
     priv->last_button_event = event->type;
-    hildon_grid_activate_child(grid, HILDON_GRID_ITEM(child));
+
+    /* If this is not the first click in this element, activate it */
+    if (already_selected)
+      hildon_grid_activate_child(grid, HILDON_GRID_ITEM(child));
 
     return FALSE;
 }
@@ -1775,6 +1818,7 @@ static gboolean jump_scrollbar_to_focused(HildonGrid * grid)
     empty_grids = priv->num_columns * priv->area_rows - child_count +
         priv->first_index;
 
+    /* Determine the position of the new row */ 
     if (priv->focus_index < priv->first_index) {
         new_row = priv->focus_index / priv->num_columns;
     } else if (priv->focus_index >= priv->first_index +
@@ -1860,6 +1904,9 @@ static gboolean adjust_scrollbar_height(HildonGrid * grid)
 
         priv->first_index = 0;
         if (GTK_WIDGET_VISIBLE(priv->scrollbar)) {
+            GtkWidget *parent = gtk_widget_get_toplevel (GTK_WIDGET (grid));
+            if (HILDON_IS_APP (parent))
+                g_object_set (parent, "scroll-control", FALSE, NULL);
             gtk_widget_hide(priv->scrollbar);
             updated = TRUE;
         }
@@ -1878,6 +1925,9 @@ static gboolean adjust_scrollbar_height(HildonGrid * grid)
     gtk_widget_size_allocate(priv->scrollbar, &alloc);
 
     if (!GTK_WIDGET_VISIBLE(priv->scrollbar)) {
+        GtkWidget *parent = gtk_widget_get_toplevel (GTK_WIDGET (grid));
+        if (HILDON_IS_APP (parent))
+            g_object_set (parent, "scroll-control", TRUE, NULL);
         gtk_widget_show(priv->scrollbar);
         updated = TRUE;
     }
@@ -1926,12 +1976,20 @@ get_child_index_by_coord(HildonGridPrivate * priv, gint x, gint y)
     int xgap, ygap;
     int t;
 
+    if (priv->item_width==0 || priv->item_height==0) {
+      return -1;
+    }
+
     xgap = x % priv->item_width;
     ygap = y % priv->item_height;
 
     if (xgap > priv->item_width - priv->h_margin) { /*FIXME*/
         return -1;
     }
+    
+    /* Event may come from outside of the grid. Skipping those events */
+    if (x >= priv->item_width * priv->num_columns)
+        return -1;
 
     t = y / priv->item_height * priv->num_columns +
         x / priv->item_width + priv->first_index;
@@ -2007,7 +2065,7 @@ static gint get_child_index(HildonGridPrivate * priv, GtkWidget * child)
  * @grid:   #HildonGrid
  * @item:   #HildonGridItem
  *
- * Emits a signal to tell HildonGridItem was actiavated.
+ * Sends a signal to indicate that this HildonGridItem is activated.
  */
 void hildon_grid_activate_child(HildonGrid * grid, HildonGridItem * item)
 {
@@ -2021,7 +2079,7 @@ void hildon_grid_activate_child(HildonGrid * grid, HildonGridItem * item)
 /**
  * hildon_grid_set_style:
  * @grid:       #HildonGrid
- * @style_name: Style name
+ * @style_name: style name
  *
  * Sets style. Setting style sets widget size, spacing, label position,
  * number of columns, and icon size.
@@ -2055,7 +2113,7 @@ void hildon_grid_set_style(HildonGrid * grid, const gchar * style_name)
  *
  * Returns the name of style currently used in HildonGrid.
  *
- * Return value: Style name
+ * Returns: style name
  */
 const gchar *hildon_grid_get_style(HildonGrid * grid)
 {
@@ -2068,8 +2126,8 @@ const gchar *hildon_grid_get_style(HildonGrid * grid)
  * get_style_properties:
  * @grid:   #HildonGrid
  *
- * Gets widget size and other stuff from gtkrc. If some stuff changed, let
- * children know this, too.
+ * Gets widget size and other properties from gtkrc. If some properties
+ * have changed, notify children of this, too.
  */
 static void get_style_properties(HildonGrid * grid)
 {
@@ -2145,6 +2203,8 @@ void hildon_grid_set_scrollbar_pos(HildonGrid * grid, gint scrollbar_pos)
 
     gtk_range_set_adjustment(GTK_RANGE(priv->scrollbar), adjustment);
 
+    g_object_notify (G_OBJECT (grid), "scrollbar-position");
+
     /* If grid isn't drawable, updating anything could mess up focus. */
     if (!GTK_WIDGET_DRAWABLE(GTK_WIDGET(grid)))
         return;
@@ -2156,9 +2216,7 @@ void hildon_grid_set_scrollbar_pos(HildonGrid * grid, gint scrollbar_pos)
  * hildon_grid_get_scrollbar_pos:
  * @grid:   #HildonGrid
  *
- * Returns position of scrollbar (in pixels).
- *
- * Return value: Scrollbar position
+ * Returns: position of scrollbar (in pixels).
  */
 gint hildon_grid_get_scrollbar_pos(HildonGrid * grid)
 {
@@ -2186,6 +2244,14 @@ hildon_grid_set_property(GObject * object,
         hildon_grid_set_empty_label(grid, g_value_get_string(value));
         break;
 
+    case PROP_STYLE:
+        hildon_grid_set_style(grid, g_value_get_string(value));
+        break;
+
+    case PROP_SCROLLBAR_POS:
+        hildon_grid_set_scrollbar_pos(grid, g_value_get_int(value));
+        break;
+
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -2196,11 +2262,21 @@ static void
 hildon_grid_get_property(GObject * object,
                          guint prop_id, GValue * value, GParamSpec * pspec)
 {
+    HildonGrid *grid;
+
+    grid = HILDON_GRID(object);
+
     switch (prop_id) {
     case PROP_EMPTY_LABEL:
-        g_value_set_string(value,
-                           hildon_grid_get_empty_label(HILDON_GRID
-                                                        (object)));
+        g_value_set_string(value, hildon_grid_get_empty_label(grid));
+        break;
+
+    case PROP_STYLE:
+        g_value_set_string(value, hildon_grid_get_style(grid));
+        break;
+
+    case PROP_SCROLLBAR_POS:
+        g_value_set_int(value, hildon_grid_get_scrollbar_pos(grid));
         break;
 
     default:
