@@ -22,11 +22,16 @@
  *
  */
 
-/* HILDON DOC
- * @shortdesc: NamePasswordDialog is a dialog for getting the user name
- * and password.
- * @longdesc: NamePasswordDialog is a simple dummy dialog for getting
- * name&password from the user.
+/**
+ * SECTION:hildon-name-password-dialog
+ * @short_description: A widget which allows a user to enter an username
+ * and a password
+ * @see_also: #HildonGetPasswordDialog, #HildonSetPasswordDialog
+ *
+ * #HildonNamePasswordDialog is used to enter a username and password
+ * when accessing a password protected function. The widget performs no
+ * input checking and is used only for retrieving a user name and a
+ * password. 
  */
 
 #include <glib.h>
@@ -57,10 +62,11 @@ struct _HildonNamePasswordDialogPrivate {
   GtkButton *closeButton;
   
   GtkLabel *domainLabel;
-  HildonCaption *nameEntry;
-  HildonCaption *passwordEntry;
+  GtkEntry *nameEntry;
+  GtkEntry *passwordEntry;
 };
 
+/* Macro to access the private data of the object instance */
 #define HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(o)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), HILDON_TYPE_NAME_PASSWORD_DIALOG,\
     HildonNamePasswordDialogPrivate))
@@ -83,35 +89,34 @@ static void hildon_name_password_dialog_get_property(GObject * object,
                                                      guint prop_id,
                                                      GValue * value, 
 						     GParamSpec * pspec);
+
 static void
 hildon_name_password_dialog_set_property(GObject * object,
                                          guint prop_id,
                                          const GValue * value, GParamSpec * pspec)
 {
-    HildonNamePasswordDialog *dialog = HILDON_NAME_PASSWORD_DIALOG(object);
-    HildonNamePasswordDialogPrivate *priv;
+    HildonNamePasswordDialog        *dialog = NULL;
+    HildonNamePasswordDialogPrivate *priv   = NULL;
 
-    priv = HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
+    dialog = HILDON_NAME_PASSWORD_DIALOG(object);
+    priv   = HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
     
     switch (prop_id) {
     case PROP_CONTENT:
-	gtk_label_set_text(priv->domainLabel, g_value_get_string(value));
-	break;
+      /* Set the password domain text */
+      hildon_name_password_dialog_set_domain(dialog, g_value_get_string(value));
+      break;
     case PROP_NAME:
-      gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child
-				   (GTK_BIN(
-					    priv->nameEntry))), 
-			 g_value_get_string(value));
+      /* Set the current username displayed in the dialog */
+      gtk_entry_set_text(priv->nameEntry, g_value_get_string(value));
       break;
     case PROP_PASSWORD:
-      gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child
-				   (GTK_BIN(
-					    priv->passwordEntry))), 
-			 g_value_get_string(value));
-        break;
+      /* Set the currently entered password */
+      gtk_entry_set_text(priv->passwordEntry, g_value_get_string(value));
+      break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
-        break;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+      break;
     }
 }
 
@@ -120,35 +125,28 @@ hildon_name_password_dialog_get_property(GObject * object,
                                          guint prop_id,
                                          GValue * value, GParamSpec * pspec)
 {
-    HildonNamePasswordDialog *dialog = HILDON_NAME_PASSWORD_DIALOG(object);
-    HildonNamePasswordDialogPrivate *priv;
-    const gchar *string;
+    HildonNamePasswordDialog        *dialog = NULL;
+    HildonNamePasswordDialogPrivate *priv   = NULL;
 
-    priv = HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
+    dialog = HILDON_NAME_PASSWORD_DIALOG(object);
+    priv   = HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
     
     switch (prop_id) {
     case PROP_CONTENT:
-      string = gtk_label_get_text(priv->domainLabel);
-      g_value_set_string(value, string);
+      g_value_set_string(value, gtk_label_get_text(priv->domainLabel));
       break;
     case PROP_NAME:
-      string = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child
-					    (GTK_BIN(
-						     priv->nameEntry))));
-
-      g_value_set_string(value, string);
+      g_value_set_string(value, hildon_name_password_dialog_get_name(dialog));
       break;
     case PROP_PASSWORD:
-     string = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child
-					    (GTK_BIN(
-						     priv->passwordEntry))));
-        g_value_set_string(value, string);
-        break;
+      g_value_set_string(value, hildon_name_password_dialog_get_password(dialog));
+     break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
-        break;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+      break;
     }
 }
+
 static void
 hildon_name_password_dialog_class_init(HildonNamePasswordDialogClass *class)
 {
@@ -156,10 +154,11 @@ hildon_name_password_dialog_class_init(HildonNamePasswordDialogClass *class)
     
     parent_class = g_type_class_peek_parent(class);
     
+    /* Override virtual functions */
     object_class->set_property = hildon_name_password_dialog_set_property;
     object_class->get_property = hildon_name_password_dialog_get_property;
     
-    /* properties */
+    /* Install new properties */
     g_object_class_install_property(object_class, 
 		    PROP_CONTENT, 
 		    g_param_spec_string ("content",
@@ -184,6 +183,7 @@ hildon_name_password_dialog_class_init(HildonNamePasswordDialogClass *class)
                             "DEFAULT",
                             G_PARAM_READWRITE));
 
+    /* Install private data structure */
     g_type_class_add_private(class,
                              sizeof(HildonNamePasswordDialogPrivate));
 }
@@ -191,17 +191,25 @@ hildon_name_password_dialog_class_init(HildonNamePasswordDialogClass *class)
 static void
 hildon_name_password_dialog_init(HildonNamePasswordDialog * dialog)
 {
+    /* Access private structure */
     HildonNamePasswordDialogPrivate *priv =
         HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
+
+    /* Size group for captions */
     GtkSizeGroup *group =
         GTK_SIZE_GROUP(gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL));
-    GtkWidget *control;
+    HildonCaption *caption;
     
+    /* Initialize dialog */
     gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_window_set_title(GTK_WINDOW(dialog), _(HILDON_NAME_PASSWORD_DIALOG_TITLE));
-    
+
+    /* Optional domain name label */    
     priv->domainLabel = GTK_LABEL(gtk_label_new(NULL));
-    
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
+	  GTK_WIDGET(priv->domainLabel), FALSE, FALSE, 0);
+
+    /* Create buttons */    
     priv->okButton =
       GTK_BUTTON(gtk_dialog_add_button(GTK_DIALOG(dialog),
                                        _(HILDON_NAME_PASSWORD_DIALOG_OK),
@@ -211,36 +219,38 @@ hildon_name_password_dialog_init(HildonNamePasswordDialog * dialog)
                                        _(HILDON_NAME_PASSWORD_DIALOG_CANCEL),
                                        GTK_RESPONSE_CANCEL));
 
-    control = gtk_entry_new();
-    priv->nameEntry = HILDON_CAPTION(hildon_caption_new
+    /* Setup user name entry */
+    priv->nameEntry = GTK_ENTRY(gtk_entry_new());
+    caption = HILDON_CAPTION(hildon_caption_new
 				     (group,
 				      _(HILDON_NAME_PASSWORD_DIALOG_NAME ),
-				      control, NULL,
+				      GTK_WIDGET(priv->nameEntry), NULL,
 				      HILDON_CAPTION_OPTIONAL));
-    hildon_caption_set_separator(priv->nameEntry, "");
+    hildon_caption_set_separator(caption, "");
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
+                       GTK_WIDGET(caption), FALSE, FALSE, 0);
 
-    control = gtk_entry_new();
-    gtk_entry_set_visibility(GTK_ENTRY(control), FALSE);
-    priv->passwordEntry =
+    /* Setup password entry */
+    priv->passwordEntry = GTK_ENTRY(gtk_entry_new());
+    gtk_entry_set_visibility(GTK_ENTRY(priv->passwordEntry), FALSE);
+    caption =
       HILDON_CAPTION(hildon_caption_new(group,
                                         _(HILDON_NAME_PASSWORD_DIALOG_PASSWORD),
-                                        control,
+                                        GTK_WIDGET(priv->passwordEntry),
                                         NULL,
                                         HILDON_CAPTION_OPTIONAL));
-    hildon_caption_set_separator(priv->passwordEntry, "");
-
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
-	  GTK_WIDGET(priv->domainLabel), FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
-                       GTK_WIDGET(priv->nameEntry), FALSE, FALSE, 0);
+    hildon_caption_set_separator(caption, "");
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
-                       GTK_WIDGET(priv->passwordEntry), FALSE, FALSE, 0);
+                       GTK_WIDGET(caption), FALSE, FALSE, 0);
 
     gtk_widget_show_all(GTK_DIALOG(dialog)->vbox);
     gtk_widget_show_all(GTK_DIALOG(dialog)->action_area);
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
     
+    /* Ensure group is freed when all its contents have been removed */
+    g_object_unref(group);
 }
+
 GType hildon_name_password_dialog_get_type(void)
 {
     static GType dialog_type = 0;
@@ -267,12 +277,12 @@ GType hildon_name_password_dialog_get_type(void)
 
 /**
  * hildon_name_password_dialog_new:
- * @parent: the parent window of the dialog.
+ * @parent: the parent window of the dialog
  *
  * Creates a new #HildonNamePasswordDialog widget with Ok and Close
  * buttons.
  *
- * Return value: the newly created #HildonNamePasswordDialog
+ * Returns: the newly created #HildonNamePasswordDialog
  */
 GtkWidget *hildon_name_password_dialog_new(GtkWindow * parent)
 {
@@ -286,21 +296,18 @@ GtkWidget *hildon_name_password_dialog_new(GtkWindow * parent)
 
 /**
  * hildon_name_password_dialog_new_with_default:
- * @parent: the parent window of the dialog.
- * @name: default name, nul-terminated string, if you want
- * to leave it unset, pass a NULL pointer.
- * @password: default password, nul-terminated string, 
- * if you want to leave it unset, pass a NULL pointer.
+ * @parent: the parent window of the dialog
+ * @name: default name, NULL if unset
+ * @password: default password, NULL if unset
  * 
  * Same as #hildon_name_password_dialog_new, but with a 
  * default name and password.
  *
- * Return value: the newly created #HildonNamePasswordDialog
+ * Returns: the newly created #HildonNamePasswordDialog
  */
-GtkWidget *hildon_name_password_dialog_new_with_default
-                                          (GtkWindow * parent,
-					   gchar *name,
-					   gchar *password)
+GtkWidget *hildon_name_password_dialog_new_with_default(GtkWindow   *parent,
+					                const gchar *name,
+					                const gchar *password)
 {
     GtkWidget *self = hildon_name_password_dialog_new(parent);
 
@@ -314,55 +321,59 @@ GtkWidget *hildon_name_password_dialog_new_with_default
 
 /**
  * hildon_name_password_dialog_get_name:
- * @dialog: the dialog.
+ * @dialog: the dialog
  *
  * Gets the text that's in the name entry.
  *
- * Return value: a pointer to the name string.
+ * Returns: a pointer to the name string.
  */
 const gchar *hildon_name_password_dialog_get_name(HildonNamePasswordDialog
                                                   * dialog)
 {
-    HildonNamePasswordDialogPrivate *priv =
-        HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
-    return gtk_entry_get_text(GTK_ENTRY(
-			      (gtk_bin_get_child
-			       (GTK_BIN(
-				priv->nameEntry)))));
+    HildonNamePasswordDialogPrivate *priv;
+
+    g_return_val_if_fail(HILDON_IS_NAME_PASSWORD_DIALOG(dialog), NULL);
+
+    priv = HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
+
+    return gtk_entry_get_text(priv->nameEntry);
 }
 
 /**
  * hildon_name_password_dialog_get_password:
- * @dialog: the dialog.
+ * @dialog: the dialog
  * 
  * Gets the text that's in the password entry.
  * 
- * Return value: a pointer to the password string.
+ * Returns: a pointer to the password string
  */
 const gchar *hildon_name_password_dialog_get_password(HildonNamePasswordDialog
                                                       * dialog)
 {
-    HildonNamePasswordDialogPrivate *priv =
-        HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
-    return gtk_entry_get_text(GTK_ENTRY
-			      (gtk_bin_get_child
-			       (GTK_BIN(
-				(priv->passwordEntry)))));
+    HildonNamePasswordDialogPrivate *priv;
+
+    g_return_val_if_fail(HILDON_IS_NAME_PASSWORD_DIALOG(dialog), NULL);
+
+    priv = HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
+
+    return gtk_entry_get_text(priv->passwordEntry);
 }
 
 /**
  * hildon_name_password_dialog_set_domain(GtkWidget *dialog, 
  * @dialog: the dialog
- * @domain: the domain or some other descriptive text to be set.
+ * @domain: the domain or some other descriptive text to be set
  * 
  * sets the optional descriptive text
  */
 
 void hildon_name_password_dialog_set_domain(HildonNamePasswordDialog *dialog, 
-                                                gchar *domain)
+					    const gchar *domain)
 {
-  HildonNamePasswordDialogPrivate *priv =
-    HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
+  HildonNamePasswordDialogPrivate *priv;
+
+  g_return_if_fail(HILDON_IS_NAME_PASSWORD_DIALOG(dialog));
+
+  priv = HILDON_NAME_PASSWORD_DIALOG_GET_PRIVATE(dialog);
   gtk_label_set_text(priv->domainLabel, domain);
-  
 }
