@@ -22,13 +22,14 @@
  *
  */
 
-/*
- * @file hildon-volumebar.c
- *
- * This file contains the implementation of the Hildon Volumebar.
- * It is a base class for Hildon Vertical Volumebar and for
- * Hildon Horizontal Volumebar classes.
+/**
+ * SECTION:hildon-volumebar
+ * @short_description: Base class for widgets that display a volume bar
+ * @see_also: #HildonHVolumebar, #HildonVVolumebar
  *
+ * #HildonVolumebar is a base class for widgets that display a volume bar that
+ * allows increasing or decreasing volume within a predefined range, and muting
+ * the volume when users click the mute icon.
  */
 
 #include <gtk/gtkwindow.h>
@@ -114,9 +115,10 @@ hildon_volumebar_get_type(void)
 static void 
 hildon_volumebar_class_init(HildonVolumebarClass *volumebar_class)
 {
-    GtkContainerClass *container_class =
-        GTK_CONTAINER_CLASS(volumebar_class);
-    GObjectClass *object_class = G_OBJECT_CLASS(volumebar_class);
+    GObjectClass      *gobject_class   = G_OBJECT_CLASS  (volumebar_class);
+    GtkObjectClass    *object_class    = GTK_OBJECT_CLASS(volumebar_class);
+    GtkWidgetClass    *widget_class    = GTK_WIDGET_CLASS(volumebar_class);
+    GtkContainerClass *container_class = GTK_CONTAINER_CLASS(volumebar_class);
 
     parent_class = g_type_class_peek_parent(volumebar_class);
 
@@ -127,8 +129,8 @@ hildon_volumebar_class_init(HildonVolumebarClass *volumebar_class)
        override forall method */
     volumebar_class->mute_toggled = mute_toggled;
     container_class->forall = hildon_child_forall;
-    GTK_WIDGET_CLASS(volumebar_class)->key_press_event = hildon_volumebar_key_press;
-    GTK_OBJECT_CLASS(volumebar_class)->destroy = hildon_volumebar_destroy;
+    widget_class->key_press_event = hildon_volumebar_key_press;
+    object_class->destroy = hildon_volumebar_destroy;
 
     signals[MUTE_TOGGLED_SIGNAL] = g_signal_new("mute_toggled",
                                                 G_OBJECT_CLASS_TYPE
@@ -153,11 +155,11 @@ hildon_volumebar_class_init(HildonVolumebarClass *volumebar_class)
                                                  gtk_marshal_VOID__VOID,
                                                  G_TYPE_NONE, 0);
     
-    object_class->set_property = hildon_volumebar_set_property;
-    object_class->get_property = hildon_volumebar_get_property; 
+    gobject_class->set_property = hildon_volumebar_set_property;
+    gobject_class->get_property = hildon_volumebar_get_property; 
 
     /*This kind of property could be usefull in the gtkcontainer*/
-    g_object_class_install_property(object_class,
+    g_object_class_install_property(gobject_class,
                                     PROP_HILDON_FOCUSABLE, 
                                     g_param_spec_boolean("can-focus",
                                     "The widget focusablility",
@@ -165,7 +167,7 @@ hildon_volumebar_class_init(HildonVolumebarClass *volumebar_class)
                                     TRUE,
                                     G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
 
-    g_object_class_install_property(object_class,
+    g_object_class_install_property(gobject_class,
                                     PROP_HILDON_HAS_MUTE, 
                                     g_param_spec_boolean("has_mute",
                                     "Show/Hide the mute button",
@@ -173,7 +175,7 @@ hildon_volumebar_class_init(HildonVolumebarClass *volumebar_class)
                                     TRUE,
                                     G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
 
-    g_object_class_install_property(object_class,
+    g_object_class_install_property(gobject_class,
 				    PROP_HILDON_LEVEL,
 				    g_param_spec_double("level",
 							"Level",
@@ -183,15 +185,13 @@ hildon_volumebar_class_init(HildonVolumebarClass *volumebar_class)
 							50.0,
 							G_PARAM_READWRITE));
 
-    g_object_class_install_property(object_class,
+    g_object_class_install_property(gobject_class,
 				    PROP_HILDON_MUTE,
 				    g_param_spec_boolean("mute",
 							 "Mute",
 							 "Whether volume is muted",
 							 FALSE,
 							 G_PARAM_READWRITE));
-				    
-
 }
 
 static void 
@@ -205,7 +205,8 @@ hildon_volumebar_init(HildonVolumebar * volumebar)
        GtkContainer */
     GTK_WIDGET_SET_FLAGS(GTK_WIDGET(volumebar), GTK_NO_WINDOW);
     GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(volumebar), GTK_CAN_FOCUS);
-    
+
+    /* Initialize mute button */
     priv->tbutton = GTK_TOGGLE_BUTTON(gtk_toggle_button_new());
     g_object_set (G_OBJECT (priv->tbutton), "can-focus", FALSE, NULL);
 }
@@ -215,17 +216,18 @@ hildon_child_forall(GtkContainer * container,
                     gboolean include_internals,
                     GtkCallback callback, gpointer callback_data)
 {
-    HildonVolumebar *vbar;
     HildonVolumebarPrivate *priv;
 
-    vbar = HILDON_VOLUMEBAR(container);
-    priv = HILDON_VOLUMEBAR_GET_PRIVATE(vbar);
+    g_assert(HILDON_IS_VOLUMEBAR(container));
+    g_assert(callback != NULL);
 
-    g_return_if_fail(callback != NULL);
+    priv = HILDON_VOLUMEBAR_GET_PRIVATE(container);
 
+    /* No external children */
     if (!include_internals)
         return;
 
+    /* Execute callback for both internals */
     (*callback) (GTK_WIDGET(priv->tbutton), callback_data);
     (*callback) (GTK_WIDGET(priv->volumebar), callback_data);
 }
@@ -256,13 +258,13 @@ hildon_volumebar_set_property(GObject * object,
                               const GValue * value, 
                               GParamSpec * pspec)
 {  
-    HildonVolumebar *vbar = HILDON_VOLUMEBAR(object);
     HildonVolumebarPrivate *priv;
 
-    priv = HILDON_VOLUMEBAR_GET_PRIVATE(vbar)
+    priv = HILDON_VOLUMEBAR_GET_PRIVATE(object);
 
     switch (prop_id) {
     case PROP_HILDON_HAS_MUTE:
+        /* Mute button always exists, but might be hidden */
         if (g_value_get_boolean(value))
             gtk_widget_show(GTK_WIDGET(priv->tbutton));
         else
@@ -316,9 +318,9 @@ hildon_volumebar_get_property(GObject * object,
 
 /**
  * hildon_volumebar_level_change:
- * @self: a HildonVolumebar widget.
+ * @self: a #HildonVolumebar widget
  *
- * Emits "level_changed" signal to the given volumebar. This function
+ * Emits "level_changed" signal to the given volume bar. This function
  * is mainly used by derived classes.
  */
 void 
@@ -330,40 +332,37 @@ hildon_volumebar_level_change(HildonVolumebar * self)
 
 /**
  * hildon_volumebar_set_level:
- * @self: volumebar to change level on
+ * @self: volume bar to change level on
  * @level: new level
  *
- * Sets new volumelevel for this #HildonVolumebar.
- **/
+ * Sets new volume level for this #HildonVolumebar.
+ */
 void 
 hildon_volumebar_set_level(HildonVolumebar * self, gdouble level)
 {
     HildonVolumebarPrivate *priv;
 
-    g_return_if_fail(self);
+    g_return_if_fail(HILDON_IS_VOLUMEBAR(self));
 
     priv = HILDON_VOLUMEBAR_GET_PRIVATE(self);
    
-    /* No need to clamp the level to legal values here as volumebarrange
-     * will do it anyway. And here we don't know the correct values anyway.
-     */
     hildon_volumebar_range_set_level(priv->volumebar, level);
 }
 
 /**
  * hildon_volumebar_get_level:
- * @self: volumebar to query level on
+ * @self: volume bar to query level on
  *
- * Gets the volumelevel of this #HildonVolumebar.
+ * Gets the volume level of this #HildonVolumebar.
  *
- * Return value: Volume level.
- **/
+ * Returns: volume level or -1 on error
+ */
 gdouble 
 hildon_volumebar_get_level(HildonVolumebar * self)
 {
     HildonVolumebarPrivate *priv;
 
-    g_return_val_if_fail(self, -1);
+    g_return_val_if_fail(HILDON_IS_VOLUMEBAR(self), -1);
 
     priv = HILDON_VOLUMEBAR_GET_PRIVATE(self);
 
@@ -372,28 +371,29 @@ hildon_volumebar_get_level(HildonVolumebar * self)
 
 /**
  * hildon_volumebar_set_mute:
- * @self: volumebar to work on
+ * @self: volume bar to work on
  * @mute: mute ON/OFF
  *
  * Sets mute status for this #HildonVolumebar.
- *
- **/
+ */
 void 
 hildon_volumebar_set_mute(HildonVolumebar * self, gboolean mute)
 {
     HildonVolumebarPrivate *priv;
     gboolean focusable = TRUE;
     
-    g_return_if_fail(self);
+    g_return_if_fail(HILDON_IS_VOLUMEBAR(self));
 
     priv = HILDON_VOLUMEBAR_GET_PRIVATE(self);
+
+    /* Slider should be insensitive when mute is on */
     gtk_widget_set_sensitive(GTK_WIDGET(priv->volumebar), !mute);
     
     focusable = GTK_WIDGET_CAN_FOCUS (GTK_WIDGET (priv->volumebar));
     
     if (mute){   
         if (focusable){
-
+	    /* Make mute button focusable since the slider isn't anymore */
             g_object_set (G_OBJECT (priv->tbutton), "can-focus", TRUE, NULL);
             gtk_widget_grab_focus (GTK_WIDGET(priv->tbutton));
         }
@@ -402,16 +402,19 @@ hildon_volumebar_set_mute(HildonVolumebar * self, gboolean mute)
     {
         g_object_set (G_OBJECT (priv->tbutton), "can-focus", FALSE, NULL);
         
+	/* Mute off grabs focus */
         if (focusable){
             gtk_widget_grab_focus (GTK_WIDGET (self));
         }
         else{
+	    /* If volumebar is not focusable, focus the parent window instead */
             GtkWidget *win = gtk_widget_get_ancestor (GTK_WIDGET (self), 
                                                       GTK_TYPE_WINDOW);
             gtk_window_set_focus (GTK_WINDOW (win), NULL);
         }
     }
 
+    /* Update mute button state and redraw */
     gtk_toggle_button_set_active(priv->tbutton, mute);
 
     gtk_widget_queue_draw (GTK_WIDGET (self));
@@ -419,18 +422,18 @@ hildon_volumebar_set_mute(HildonVolumebar * self, gboolean mute)
 
 /**
  * hildon_volumebar_get_mute:
- * @self: volumebar to query mute status
+ * @self: volume bar to query mute status
  *
  * Gets mute status of this #HildonVolumebar (ON/OFF).
  *
- * Return value: Mute status as #gboolean value.
- **/
+ * Returns: Mute status as #gboolean value.
+ */
 gboolean 
 hildon_volumebar_get_mute(HildonVolumebar * self)
 {
     HildonVolumebarPrivate *priv;
 
-    g_return_val_if_fail(self, TRUE);
+    g_return_val_if_fail(HILDON_IS_VOLUMEBAR(self), TRUE);
 
     priv = HILDON_VOLUMEBAR_GET_PRIVATE(self);
 
@@ -439,24 +442,24 @@ hildon_volumebar_get_mute(HildonVolumebar * self)
 
 /**
  * hildon_volumebar_get_adjustment
- * @self : A #HildonVolumebar
+ * @self : a #HildonVolumebar
  * 
- * Gets the GtkAdjustment used in volumebar. This can be handy
+ * Gets the GtkAdjustment used in volume bar. This can be handy
  * to give to hildon_appview_set_connected_adjustment which
  * will allow changing the volume with increase / decrease
  * hardware buttons.
  *
- * This is a temporary solution until volumebar is restructured to
+ * This is a temporary solution until volume bar is restructured to
  * be a child class of GtkRange.
  * 
- * Return value: A @GtkAdjustment used by volumebar.
- **/
+ * Returns: a #GtkAdjustment used by volume bar.
+ */
 GtkAdjustment * 
 hildon_volumebar_get_adjustment (HildonVolumebar * self)
 {
     HildonVolumebarPrivate *priv;
 
-    g_return_val_if_fail(self, NULL);
+    g_return_val_if_fail(HILDON_IS_VOLUMEBAR(self), NULL);
 
     priv = HILDON_VOLUMEBAR_GET_PRIVATE(self);
 
@@ -466,7 +469,9 @@ hildon_volumebar_get_adjustment (HildonVolumebar * self)
 static void
 mute_toggled (HildonVolumebar *self)
 {
-  /* only call hildon_volumebar_set_mute. everything is done there */
+  /* This looks like no-op, but it still does something meaningfull!
+     set_mute also updates the ui to match new state that
+     is already reported by get_mute */
   hildon_volumebar_set_mute (self, hildon_volumebar_get_mute(self));
 }
 
@@ -476,11 +481,9 @@ hildon_volumebar_key_press (GtkWidget * widget,
 {
     HildonVolumebarPrivate *priv;
     
-    g_return_val_if_fail(widget, FALSE);
-    g_return_val_if_fail(event, FALSE);
-
     priv = HILDON_VOLUMEBAR_GET_PRIVATE(widget);
-    
+
+    /* Enter key toggles mute button (unless it is hidden) */
     if (event->keyval == GDK_Return && GTK_WIDGET_VISIBLE(priv->tbutton)) {
         gtk_toggle_button_set_active(priv->tbutton, 
                 !hildon_volumebar_get_mute(HILDON_VOLUMEBAR(widget)));
@@ -489,3 +492,12 @@ hildon_volumebar_key_press (GtkWidget * widget,
 
     return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget, event);
 }
+
+/* Sends mute-toggled signal to widget, used as a callback in derived classes
+   Just keep this "protected" in order to avoid introducing new API. */
+void
+_hildon_volumebar_mute_toggled(HildonVolumebar * self)
+{
+    g_return_if_fail(HILDON_IS_VOLUMEBAR(self));
+    g_signal_emit_by_name(self, "mute_toggled");
+}
